#!/bin/bash

# Add a file to the list of dependencies if it's not already there
function add_to_dependencies {
	ADD_TO_DEPENDENCIES_FOUND=0
	for lib in $DEPENDENCIES
	do
		if [ "$lib" == "$1" ]
		then
			ADD_TO_DEPENDENCIES_FOUND=1
			break
		fi
	done
	if [ $ADD_TO_DEPENDENCIES_FOUND == 0 ]
	then
		DEPENDENCIES="$DEPENDENCIES $1"
		HAVE_ADDED=1
	fi
}

# Given a list of binaries, append the list with all binaries they depend on according to ldd (and the dependencies of those, etc.)
function get_dependencies {
	DEPENDENCIES=$@
	CHECKED=
	HAVE_ADDED=1

	while [ $HAVE_ADDED == 1 ]
	do
		HAVE_ADDED=0
		for lib in $DEPENDENCIES
		do
			GET_DEPENDENCIES_FOUND=0
			for checkedlib in $CHECKED
			do
				if [ "$checkedlib" == "$lib" ]
				then
					GET_DEPENDENCIES_FOUND=1
					break
				fi
			done
			if [ $GET_DEPENDENCIES_FOUND == 0 ]
			then
				CHECKED="$CHECKED $lib"
				NEWDEPS="`ldd $lib | awk '{if($2 == "=>") print $3; else if($1 != "statically" && $4 != "executable") print $1}'`"
				for newlib in $NEWDEPS
				do
					add_to_dependencies $newlib
				done
			fi
		done
	done
}

TARGET=`mktemp -d`
TMP_IMG=`mktemp`
BINARIES="/bin/bash /bin/mknod /sbin/fsck.hfsplus /sbin/fsck.ext2 /bin/mount /sbin/losetup /bin/mkdir /bin/cat /usr/bin/tr /bin/sleep /bin/chmod /usr/sbin/chroot /sbin/pivot_root /bin/echo /bin/ls /bin/umount"
get_dependencies $BINARIES

# Create the image
dd if=/dev/zero of=$TMP_IMG bs=1 seek=4999999 count=1 &> /dev/null
mkfs.ext2 -F $TMP_IMG &> /dev/null
LOOP=`losetup -f`
losetup $LOOP $TMP_IMG &> /dev/null
mount $LOOP $TARGET

# Create the initial tree
cp --preserve --dereference --parents --target $TARGET $DEPENDENCIES

# Create a symlink for sh (optional)
ln -s bash $TARGET/bin/sh

# Copy firmware
mkdir -p $TARGET/lib/firmware
cp /lib/firmware/* $TARGET/lib/firmware

# Copy the init script
cat > $TARGET/init << INIT_IS_DONE
#!/bin/bash

[ -d /dev ] || /bin/mkdir -m 0755 /dev
[ -d /root ] || /bin/mkdir --mode=0700 /root
[ -d /sys ] || /bin/mkdir /sys
[ -d /proc ] || /bin/mkdir /proc
[ -d /tmp ] || /bin/mkdir /tmp
[ -d /etc ] || /bin/mkdir /etc
[ -d /mnt ] || /bin/mkdir /mnt
[ -d /android ] || /bin/mkdir /android

[ -e /dev/console ] || /bin/mknod /dev/console c 5 1

cat > /etc/fstab << FSTAB_DONE
FSTAB_DONE

/bin/mount -t sysfs none /sys -onodev,noexec,nosuid
/bin/mount -t proc none /proc -onodev,noexec,nosuid

# We need to load the firmware as quickly as possible because there's a timeout
while [ ! -e /sys/class/firmware/mmc*/loading ]
do
	/bin/sleep 1
done

# Load the helper
echo 1 > /sys/class/firmware/mmc*/loading
/bin/cat /lib/firmware/sd8686_helper.bin > /sys/class/firmware/mmc*/data
echo 0 > /sys/class/firmware/mmc*/loading

/bin/sleep 1

while [ ! -e /sys/class/firmware/mmc*/loading ]
do
	/bin/sleep 1
done

# Load the main firmware
echo 1 > /sys/class/firmware/mmc*/loading
/bin/cat /lib/firmware/sd8686.bin > /sys/class/firmware/mmc*/data
echo 0 > /sys/class/firmware/mmc*/loading

while [ ! -e /sys/class/block/nand0p2/dev ]
do
	/bin/sleep 1
done

/bin/mknod /dev/loop0 b 7 0 &> /dev/console
/bin/mknod /dev/loop1 b 7 1 &> /dev/console
/bin/mknod /dev/loop2 b 7 2 &> /dev/console
/bin/mknod /dev/loop3 b 7 3 &> /dev/console
/bin/mknod /dev/nand0p2 b \`/bin/cat /sys/class/block/nand0p2/dev | /usr/bin/tr ':' ' '\`

/sbin/fsck.hfsplus -q /dev/nand0p2 &> /dev/null
if [ \$? != 0 ]
then
	/sbin/fsck.hfsplus /dev/nand0p2
fi

/bin/mount -o noatime /dev/nand0p2 /mnt
/bin/chmod a+rx /mnt
/sbin/losetup /dev/loop0 /mnt/ramdisk.img
/sbin/losetup /dev/loop1 /mnt/system.img
/sbin/losetup /dev/loop2 /mnt/userdata.img
/sbin/losetup /dev/loop3 /mnt/cache.img

/sbin/fsck.ext2 -p /dev/loop0
/sbin/fsck.ext2 -p /dev/loop1
/sbin/fsck.ext2 -p /dev/loop2
/sbin/fsck.ext2 -p /dev/loop3

/bin/mount -o noatime /dev/loop0 /android
/bin/mkdir /android/host
/bin/mkdir /android/system
/bin/mkdir /android/data
/bin/mkdir /android/cache

/bin/mount -o noatime,ro /dev/loop1 /android/system
/bin/mount -o noatime,nosuid,nodev /dev/loop2 /android/data
/bin/mount -o noatime,nosuid,nodev /dev/loop3 /android/cache

#/bin/mkdir /android/initrd

/bin/umount /sys
/bin/umount /proc
/bin/mount -n --move /mnt /android/host

cd /android
#/sbin/pivot_root . initrd
exec /usr/sbin/chroot . /init

INIT_IS_DONE

chmod 0755 $TARGET/init

pushd $TARGET > /dev/null
find . -ls
popd > /dev/null

umount $LOOP
fsck.ext2 $LOOP
losetup -d $LOOP

gzip -9 -c $TMP_IMG > $1

# Move the image to the final place
rm -rf $TMP_IMG

# Delete temporary directory
rm -rf $TARGET
